# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - ReadMessage Action

    @copyright: 2012, 2013 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.action import ActionBase
from MoinSupport import *
from ItemSupport import ItemStore
from email.parser import Parser

Dependencies = []

class ReadMessage(ActionBase, ActionSupport):

    "An action that can read a stored message component."

    def __init__(self, pagename, request):

        """
        On the page with the given 'pagename', use the given 'request' to access
        message components.
        """

        ActionBase.__init__(self, pagename, request)
        self.store = ItemStore(self.page, "messages", "message-locks")

    def get_form_html(self, buttons_html):

        "Present an interface for accessing a message component."

        _ = self._
        request = self.request
        form = self.get_form()

        message = form.get("message", [""])[0]
        part = form.get("part", [""])[0]

        # Fill in the fields and labels.

        d = {
            "buttons_html"          : buttons_html,
            "message_label"         : _("Message number"),
            "message_default"       : escattr(message),
            "part_label"            : _("Part identifier"),
            "part_default"          : escattr(part),
            }

        # Prepare the output HTML.

        html = '''
<table>
    <tr>
        <td class="label"><label>%(message_label)s</label></td>
        <td>
            <input name="message" type="text" value="%(message_default)s" />
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(part_label)s</label></td>
        <td>
            <input name="part" type="text" value="%(part_default)s" />
        </td>
    </tr>
    <tr>
        <td></td>
        <td class="buttons">
            %(buttons_html)s
        </td>
    </tr>
</table>''' % d

        return html

    def do_action(self):

        "Attempt to send the message."

        _ = self._
        request = self.request
        form = self.get_form()

        message_number = form.get("message", [None])[0]
        part_identifier = form.get("part", [None])[0]

        if not message_number:
            return 0, _("A message number must be given.")

        if not part_identifier:
            return 0, _("A part identifier must be given.")

        # Obtain the message.

        try:
            message_text = self.store[int(message_number)]
        except (IndexError, ValueError):
            return 0, _("No such message is stored on this page.")

        # Visit the message parts, looking for the indicated component.

        message = Parser().parsestr(message_text)

        if message.is_multipart():
            for part in message.get_payload():

                # For the selected component, return the content as a response
                # to the current request.

                if part.get("Content-ID") == part_identifier:
                    charset = part.get_content_charset()
                    headers = [
                        "Content-Type: %s%s" % (
                            part.get_content_type(),
                            charset and ("; charset=%s" % charset) or ""
                            )
                        ]
                    get_send_headers(request)(headers)
                    request.write(part.get_payload(decode=True))
                    return 1, None

        return 0, _("No such component in the indicated message.")

    def render_success(self, msg, msgtype=None):

        """
        Render neither 'msg' nor 'msgtype' since a resource has already been
        produced.
        NOTE: msgtype is optional because MoinMoin 1.5.x does not support it.
        """

        pass

# Action function.

def execute(pagename, request):
    ReadMessage(pagename, request).render()

# vim: tabstop=4 expandtab shiftwidth=4
